import { EditorView } from '@codemirror/view';
import { HighlightStyle, syntaxHighlighting } from '@codemirror/language';
import { tags } from '@lezer/highlight';

// north theme colors adapted to the structure
const northColors = {
    background: '#200020',
    foreground: '#D0D0FF',
    caret: '#7070FF',
    selection: 'rgba(128, 0, 0, 0.5)', // Updated to rgba for clarity
    gutterBackground: '#200020',
    gutterForeground: '#C080C0',
    comment: '#ff0000',
    string: '#999999',
    number: '#7090B0',
    boolNull: '#8080A0',
    punctuation: '#805080',
    keyword: '#60B0FF',
    definitionKeyword: '#B0FFF0',
    operator: '#A0A0FF',
    variableName: '#008080',
    className: '#70E080',
    propertyName: '#50A0A0',
    tagName: '#009090',
    modifier: '#B0FFF0',
    attributeName: '#D0D0FF',
};

// The editor theme styles for the north theme.
const northTheme = EditorView.theme({
    "&": {
        color: northColors.foreground,
        backgroundColor: northColors.background
    },
    ".cm-content": {
        caretColor: northColors.caret
    },
    ".cm-cursor, .cm-dropCursor": { borderLeftColor: northColors.caret },
    "&.cm-focused > .cm-scroller > .cm-selectionLayer .cm-selectionBackground, .cm-selectionBackground, .cm-content ::selection": { backgroundColor: northColors.selection },
    ".cm-gutters": {
        backgroundColor: northColors.gutterBackground,
        color: northColors.gutterForeground,
        border: "none"
    },
    ".cm-activeLineGutter": {
        backgroundColor: northColors.selection
    },
}, { dark: true });

// The highlighting style for code in the north theme.
const northHighlightStyle = HighlightStyle.define([
    { tag: tags.comment, color: northColors.comment },
    { tag: [tags.string, tags.regexp], color: northColors.string },
    { tag: tags.number, color: northColors.number },
    { tag: [tags.bool, tags.null], color: northColors.boolNull },
    { tag: [tags.punctuation, tags.derefOperator], color: northColors.punctuation },
    { tag: tags.keyword, color: northColors.keyword },
    { tag: tags.definitionKeyword, color: northColors.definitionKeyword },
    { tag: tags.moduleKeyword, color: northColors.keyword },
    { tag: tags.operator, color: northColors.operator },
    { tag: [tags.variableName, tags.self], color: northColors.variableName },
    { tag: tags.operatorKeyword, color: northColors.operator },
    { tag: tags.controlKeyword, color: northColors.keyword },
    { tag: tags.className, color: northColors.className },
    { tag: [tags.function(tags.propertyName), tags.propertyName], color: northColors.propertyName },
    { tag: tags.tagName, color: northColors.tagName },
    { tag: tags.modifier, color: northColors.modifier },
    { tag: [tags.squareBracket, tags.attributeName], color: northColors.attributeName },
]);
console.log('HighlightStyle defined:', northHighlightStyle);

// Extension to enable the north theme (both the editor theme and the highlight style).
const north = [northTheme, syntaxHighlighting(northHighlightStyle)];
console.log('Extensions:', northTheme);

export { northTheme, northHighlightStyle, north };
